unit messagepipe;

interface

type
  TMessage = record
    SomeString: string;
  end;

type
  {$A8}
  TMessagePipe = class
  private
    fPushIndex: uint32;
    fPullIndex: uint32;
    fMessages: array of TMessage;
  public
    function Push( aMessage: TMessage ): boolean;
    function Pull( var aMessage: TMessage ): boolean;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
  end;

implementation

{ TMessagePipe }

constructor TMessagePipe.Create;
begin
  inherited Create;
  fPushIndex := 0;
  fPullIndex := 0;
  SetLength(fMessages,5);
end;

destructor TMessagePipe.Destroy;
begin
  SetLength(fMessages,0);
  inherited Destroy;
end;

function TMessagePipe.Pull(var aMessage: TMessage): boolean;
var
  NewIndex: uint32;
begin
  Result := False;
  if fPullIndex=fPushIndex then begin
    exit;
  end;
  aMessage.SomeString := fMessages[fPullIndex].SomeString;
  NewIndex := succ(fPullIndex);
  if NewIndex>=Length(fMessages) then begin
    NewIndex := 0;
  end;
  fPullIndex := NewIndex;
  Result := True;
end;

function TMessagePipe.Push(aMessage: TMessage): boolean;
var
  NewIndex: uint32;
begin
  Result := False;
  NewIndex := succ(fPushIndex);
  if (NewIndex>=Length(fMessages)) then begin
    NewIndex := 0;
  end;
  if NewIndex=fPullIndex then begin
    Exit;
  end;
  fMessages[fPushIndex].SomeString := aMessage.SomeString;
  fPushIndex := NewIndex;
  Result := True;
end;

end.
